
import becker.io.TextInput;
import becker.io.TextOutput;
import becker.util.DateTime;
import java.text.NumberFormat;

/** Model a person participating in Big Brother/Big Sister.
 * @author Byron Weber Becker */
class Person extends Object
{	private String name;		// person's name
	private DateTime birthdate;
	private char gender;		// one of {M, F}
	private char role;		// one of {B, L}
	private String pairName;	// name of the person this person is paired with
										// null if not yet paired
	private double likesCrafts = 0.0;
	private double likesGames = 0.0;
	private double likesOutdoors = 0.0;
	private double likesSports = 0.0;
	
	/** Construct a person object using information found in a text file.
	 * The format is the same as output by the write() method.
	 * @param in an open input file. */
	public Person(TextInput in)
	{	super();
		this.name = in.readLine();
		this.birthdate = new DateTime(in);
		this.gender = in.readToken().charAt(0);
		this.role = in.readToken().charAt(0);
		in.readLine();
		if (!in.doubleIsAvailable())
		{	this.pairName = in.readLine();
		}
		this.likesCrafts = in.readDouble();
		this.likesGames = in.readDouble();
		this.likesOutdoors = in.readDouble();
		this.likesSports = in.readDouble();
		in.readLine();
	}
	
	/** @return This person's name */
	public String getName()
	{	return this.name;
	}
	
	/** @return This person's age */
	public int getAge()
	{	DateTime today = new DateTime();
		int yrs = today.getYear() - this.birthdate.getYear();
		if (today.getMonth() > this.birthdate.getMonth() || 
			 today.getMonth() == this.birthdate.getMonth() && today.getDay() > this.birthdate.getDay())
		{	yrs--;
		}
		return yrs;
	}
	
	/** @return This person's birthdate */
	public DateTime getBirthdate()
	{	return this.birthdate;
	}
	
	/** @return This person's gender, one of {'M', 'F'}. */
	public char getGender()
	{	return this.gender;
	}
	
	/** @return This person's role, one of {'B', 'L'} (a "big" or a "little") */
	public char getRole()
	{	return this.role;
	}
	
	/** @return The name of the person this person is paired with or the
	 *  empty string if not yet paired. */
	public String getPairName()
	{	if (this.pairName == null)
		{	return "";
		} else 
		{	return this.pairName;
		}
	}
	
	/** @return true if this person is paired with someone else; false otherwise */
	public boolean isPaired()
	{	return this.pairName != null;
	}
	
	/** Pair this person with p.
	 * @param p The person to pair this person with. */
	public void pairWith(Person p)
	{	this.pairName = p.name;
		p.pairName = this.name;
	}
	
	/** Get the compatibility score for this person and person p.  Returns 0.0
	 * if either person is already paired, or if both are littles or bigs, or if
	 * they have different genders.
	 * @param p a potential pair for this person 
	 * @return a number between 0.0 and 1.0 where 0.0 is not at all compatible
	 *  and 1.0 is very compatible. */
	public double getCompatibility(Person p)
	{	if (this.isPaired() || p.isPaired() || this.role == p.role || this.gender != p.gender)
		{	return 0.0;
		}
		return (this.likesCrafts * p.likesCrafts + this.likesGames * p.likesGames +
				  this.likesOutdoors * p.likesOutdoors + this.likesSports * p.likesSports)/4.0;
	}
	
	/** Represent this person as a string.
	 * @return the representation as a string. */
	public String toString()
	{	String g = "Male";
		if (this.gender == 'F')
		{	g = "Female";
		}
		String r = "Little";
		if (this.role == 'B')
		{	r = "Big";
		}
		String p = "Name: " + this.name + " (" + g + ")\n" +
				 "Birthdate: " + this.birthdate + " (" + r + ")\n";
				 
		if (this.isPaired())
		{	p = p + "Paired with: " + this.pairName + "\n";
		}
		
		p = p + this.likesCrafts + " " + this.likesGames + " " + this.likesOutdoors + " " + this.likesSports + "\n";
		return p;
	}
	
	/** Write this person to a file.
	 * @param out The open output file. */
	public void write(TextOutput out)
	{	out.println(this.name);
		out.println(this.birthdate.toString() + " " + this.gender + " " + this.role);
		NumberFormat nf = NumberFormat.getNumberInstance();
		nf.setMaximumFractionDigits(1);
		nf.setMinimumFractionDigits(1);
		out.println(nf.format(this.likesCrafts) + " " + nf.format(this.likesGames) + " " +
						nf.format(this.likesOutdoors) + " " + nf.format(this.likesSports));
	}
}
