
import becker.xtras.connect4.*;
import ljwari.io.ConsoleInput;

/** A class providing a text-based interface for a game of Connect4.
 * @author Byron Weber Becker 
 **/
public class EventLoopUI extends Object
{
   private Connect4 c4;									// the game's logic
   private ConsoleInput in = new ConsoleInput();	// read from user
   private static final String PROMPT = "> ";
   private boolean done = false;						// is the user done yet?

	/** Construct a user interface object.
	 * @param theGame The game object to interface with. */
   public EventLoopUI(Connect4 theGame)
   {  super();
      this.c4 = theGame;
   }

   /** Play the game by repeatedly displaying the board, asking the user for a 
    *  move, and making that move. */
   public void playGame()
   {  this.showRules();
      while (!this.done)				// our event loop
      {  this.displayBoard();
      	char cmd = this.readCommand(EventLoopUI.PROMPT);
      	this.executeCommand(cmd);
      }
   }
   
   /** Display the board to the user. */
   private void displayBoard()
   {	System.out.print(this.c4.getBoard());
   }
      
  	/** Read a command from the user. */
  	private char readCommand(String prompt)
  	{	System.out.print(prompt);
  		String cmd = this.in.readToken();	// skip whitespace
  		return cmd.charAt(0);				// return just first char
  	}
  	
  	/** Execute the command or give an error message if not recognized. */
  	private void executeCommand(char cmd)
  	{  if ('1' <= cmd && cmd < '1' + Connect4.NUMCOLS)
	   {  this.makeMove(cmd);
	   } else if (cmd == 'n' || cmd == 'N')
	   {  this.newGame();
	   } else if (cmd == 'h' || cmd == 'H')
	   {  this.help();
	   } else if (cmd == 'r' || cmd == 'R')
	   {  this.showRules();
	   	this.in.readLine();	// can't be in showRules because it is also called above
	   } else if (cmd == 'q' || cmd == 'Q')
	   {  this.quit();
	   } else if (cmd == 'd' || cmd == 'D')
	   {  this.setDifficulty();
	   } else
	   {  this.in.readLine();
	      System.out.println("Unrecognized command: '" + cmd + "'.");
	   }
  	}
  	
   /** Quit the application. */
   private void quit()
   {  this.done = true;
   	this.in.readLine();
      System.out.println("Thank you for playing!");   	
   }

	/** Make a move -- provided the game isn't over and checking that the move
	 * is, in fact, legal.
	 * @param cmd A character in the range '1'...Connect4.NUM_COLUMNS. */
   private void makeMove(char cmd)
   {	this.in.readLine();		// read rest of the command line
   
      int col = cmd - '0';		// find column number from command
      
	   if (this.c4.isOver())
      {	System.out.println("The game is over.  Use 'n' to start a new game or 'q' to quit.");
   	} else if (!this.c4.isLegalMove(col))
   	{	System.out.println("Column " + col + " is not a legal move.  Please try again.");
	   } else
	   {  this.c4.placeToken(col);
	      if (this.c4.isOver())
	      {  System.out.println("*\n* " + this.c4.getEndOfGameMessage() + "\n*\n");
	      }
	 	}
   }

	/** Set the difficulty of the computer's player. */
   private void setDifficulty()
   {  int d = Integer.MIN_VALUE;
      if (this.in.intIsAvailable())
      {  d = this.in.readInt();      
      } 	
      this.in.readLine();
      if (d < Connect4.MIN_DIFFICULTY || d > Connect4.MAX_DIFFICULTY)
      {  d = this.in.demandInt("Difficulty", Connect4.MIN_DIFFICULTY, Connect4.MAX_DIFFICULTY);
      }
      this.c4.setDifficulty(d);
   }

/* BEGIN_OMIT */

	/** Show the rules of the game. */
   private void showRules()
   {  System.out.println(
         "   *** Connect 4 ***\n" +
         "You and the computer will take turns placing tokens\n" +
         "in the columns on the 'board'.  Tokens always fall\n" +
         "to the bottom of the column.  The first player to\n" +
         "place four tokens in a row either horizontally,\n" +
         "vertically, or diagonally is the winner.\n\n" +
         "You will play '" + Connect4.HUMAN + "' and the computer will play '" + Connect4.COMPUTER + "'.\n" +
         "Type 'h' (help) to see a list of valid commands.\n"
         );
   }

	/** Start a new game. */
   private void newGame()
   {  boolean humanFirst;
      if (this.in.tokenIsAvailable())
      {  String arg = this.in.readToken();
      	this.in.readLine();
         if (arg.equalsIgnoreCase("u"))
         {  humanFirst = true;
         } else if (arg.equalsIgnoreCase("c"))
         {  humanFirst = false;
         } else
         {  humanFirst = this.in.demandYesNo("Do you want to go first?");
         }
      } else
      {  this.in.readLine();
         humanFirst = this.in.demandYesNo("Do you want to go first?");
      }
      this.c4.initGame(humanFirst);
   }

	/** Display the available commands. */
   private void help()
   {  this.in.readLine();
		System.out.println(
         "Valid commands are:\n" +
         "   1-" + Connect4.NUMCOLS + "    Place your token in the given column\n" +
         "   n [uc]  Start a new game (u=user plays first; c=compter plays first)\n" +
         "   h       Print this help message\n" +
         "   q       Quit the program\n" +
         "   d [1-3] Set the difficulty level (1 is easy, 3 is hard)\n" +
         "   r       Show the rules\n" +
         "Characters in square brackets are optional.\n"
         );
   }
/* END_OMIT */
}