
import javax.swing.*;
import java.util.*;
import java.awt.*;
import becker.util.Test;


/** An instance of PaintablesManager manages all the Paintable objects
 * displayed in a city.  Paintables are painted in layers, starting with
 * layer 0.  Layer 0 contains the background (intersections).  Layer 1
 * contains things on the intersections (flashers, streetlights, etc), 
 * and layer 2 contains robots.
 *
 * @author Byron Weber Becker */
public class PaintablesManager extends Object
{
   private static final int INIT_LAYER_CAP = 1;      // must be > 0
   private Paintable[][] layers = new Paintable[0][];
   private int[] layerSize = new int[0];

   /** Construct an empty PaintablesManager */
   public PaintablesManager()
   {  super();
   }

   /** Paint this component by painting each of the Paintables that
    have been added to it. Go layer by layer, from layer 0 up.
    @param g The graphics object to use for painting. */
   
   /* package */ void paint(Graphics2D g2)
   {  for (int r = 0; r < this.layers.length; r++)
      {  Paintable[] layer = this.layers[r];
         for (int c = 0; c < this.layerSize[r]; c++)
         {  layer[c].paint(g2);
         }
      }
   }

   /** Add another Paintable thing to the view.
    @param p The Paintable thing to add.
    @param layer The layer it should be added to */
   public void addPaintable(Paintable p, int layer)
   {  if (layer >= this.layers.length)
      {  this.addMoreLayers(layer + 1);
      }

      int size = this.layerSize[layer];
      if (size >= this.layers[layer].length)
      {  this.enlargeLayer(layer);
      }

      this.layers[layer][this.layerSize[layer]] = p;
      this.layerSize[layer]++;
   }

   /** The given layer isn't large enough to hold all the Paintables.
    @param layer The layer that needs to be larger */
   private void enlargeLayer(int layer)
   {  Paintable[] aLayer = this.layers[layer];
      Paintable[] t = new Paintable[aLayer.length * 2];
      for (int i = 0; i < this.layerSize[layer]; i++)
      {  t[i] = aLayer[i];
      }
      this.layers[layer] = t;
   }

   /** We need another layer for the view.  Add some.
    @param max The max number of layers (so far). */
   private void addMoreLayers(int max)
   {  Paintable[][] t = new Paintable[max][];
      int[] s = new int[max];
      for (int i = 0; i < this.layers.length; i++)
      {  t[i] = this.layers[i];
         s[i] = this.layerSize[i];
      }
      for (int i = this.layers.length; i < max; i++)
      {  t[i] = new Paintable[INIT_LAYER_CAP];
         s[i] = 0;
      }
      this.layers = t;
      this.layerSize = s;
   }

   /** Testing */
   public static void main(String[] args)
   {  PaintablesManager dm = new PaintablesManager();

      Test.ckIsNotNull("view not null", dm);
      Test.ckIsNotNull("layers not null", dm.layers);
      Test.ckIsNotNull("layerSize not null", dm.layerSize);
      Test.ckEquals("num layers", 0, dm.layers.length);
      for (int i = 0; i < PaintablesManager.INIT_LAYER_CAP + 4; i++)
      {  Paintable d = new Paintable();
         dm.addPaintable(d, 1);
         Test.ckIdentical("added Paintable" + i, d, dm.layers[1][i]);
      }
      Test.ckEquals("layer 1 size", PaintablesManager.INIT_LAYER_CAP + 4,
            dm.layerSize[1]);
   }
}
